<?php
//    Copyright 2010 Andrii Salnikov
//
//   Licensed under the Apache License, Version 2.0 (the "License");
//   you may not use this file except in compliance with the License.
//   You may obtain a copy of the License at
//
//       http://www.apache.org/licenses/LICENSE-2.0
//
//   Unless required by applicable law or agreed to in writing, software
//   distributed under the License is distributed on an "AS IS" BASIS,
//   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//   See the License for the specific language governing permissions and
//   limitations under the License.
//

	date_default_timezone_set('UTC');
	require_once("miscmail.php");

	//**************** FUNCTIONS DEFINITION ***************
	// Syntax simplification of POST checking
	function _ispost ( $postindex, $defvalue = "" ) {
		return isset($_POST[$postindex])?trim($_POST[$postindex]):$defvalue;
	}

	// Posibly undefined variables handling 
	function _isdef ( &$var, $def = "N/A" ) {
		if ( isset($var) ) return $var;
		return $def;
	}
	
	// Carefully close HTML after error reporting
	function closehtml ($depth = 0) {
		global $voms_footer_caption;
		if ( $depth = 1 )
			// when error printed from #workarea
			echo "</div><div style=\"clear: both;\"></div><div id=\"footer\">" .$voms_footer_caption ."</div></div></div></body></html>";
		else
			// when error printed from menu ( default )
			echo "</div><div id=\"maincontainer\"><div id=\"footer\">". $voms_footer_caption ."</div></div></div></body></html>";
		exit(1);
	}

	// Exit with printing message about hack detection
	function hackexit ($depth = 0) {
		global $voms_hack;
		echo "<p class=\"error\">". $voms_hack ."</p>";
		closehtml($depth);
	}

	// Odd recoloring
	function doOddtrtd ( &$odd, $w=250 ) {
		if ( $odd == 1 ) {
	      		printf("<tr class=\"trodd\">\n<td style=\"width:%spx;\">",$w);
			$odd = 0;
		} else {
			printf("<tr>\n<td style=\"width:%spx;\">",$w);
			$odd = 1;
		}
	}

	// Get CN from DN
	function CNfromDN ( $dn ) {
		$cn = "";
		$dot_cn = "";
		if ( preg_match("!/CN=!", $dn) ) {
			$dot_count = $space_count = 0;
			foreach ( preg_split("!/CN=!",$dn) as $dn_field ) {
				if ( ! preg_match("/=/", $dn_field) ) {
					// Check if CN have spaces
					$s = preg_match("/\s/", $dn_field);
					if ( $s >= $space_count ) { 
						$cn = $dn_field;
						$space_count = $s;
					}
					// Check if CN have dots
					$d = preg_match("/\./", $dn_field);
					if ( $d >= $dot_count ) {
						$dot_cn = $dn_field;
						$dot_count = $d;
					}
				}
			}
			// If we have record with spaces (more spaces is preferable) - use it as CN
			// If we have no records with spaces, but have records with dots (more dots is preferable) - use it as CN
			// Otherwise just use last CN record
			if ( ($space_count === 0) && ($dot_count > 0) ) $cn = $dot_cn;
			$cn = preg_replace("/^CN=/","",$cn);
		}
		if ( $cn == '' ) $cn = $dn;
		return $cn;
	}

	// Check ACL rights
	function isACLallow ( $permissions_a, $class, $op, $quiet = 0 ) {
		global $voms_insufficient_read;
		//printf("%s-%s: %s", $class, $op, isset($permissions_a[$class][$op]));
		$result = ( isset($permissions_a[$class][$op]) ) ? $permissions_a[$class][$op] : 0;
		if ( ( $op === "r" ) && ( $result == 0 ) && ( $quiet == 0 )) printf($voms_insufficient_read, $class);
		return $result;
	}

	// Get all subgroups ACL permissions
	// Return hash: [gid] => permissions decoded array;
	function getGroupsPermissions ( $gids, $dn, $ca, $member ) {
		$gperms = array();
		foreach ( $gids as $gid ) {
			$g_perm = getProperUserACL ($dn, $ca, $member, $gid);
			if ( $g_perm == 0 ) $g_perm_a = $g1_permissions_a;
			else $g_perm_a = decodeACLPermissions($g_perm);
			$gperms[$gid] = $g_perm_a;
		}
		return $gperms;
	}

	// Input control for search function
	function showSearchInput ( $caption, $additional = null, $action = "" ) {
		$svalue = _ispost("search");
		printf("<div class=\"search\">\n<form method=\"post\" action=\"%s\">\n<input class=\"searchtxt\" type=\"text\" name=\"search\" value=\"%s\" />\n", $action, $svalue);
		printf("<input class=\"searchbt\" type=\"submit\" name=\"sbutton\" value=\"%s\" />\n", $caption);
		if ( $additional ) echo $additional . "\n";
		echo "</form>\n</div>\n";
	}

	// Limits in tables output
	function ShowLimitsCaption($hrefbase, $allmembers, $todiv = "" ) {
		global $items_per_page, $voms_tcaption_tmpl;
		global $limit, $filter;
		echo "<div class=\"tcaption\">\n";
		$limit_prev = ( ($limit - $items_per_page) >= 0 )  ? ($limit - $items_per_page) : 0;
		$limit_next = $limit + $items_per_page;
		if ( $filter !== null ) $hrefbase .= "&amp;search=".htmlspecialchars($filter);
		$tcaption = ( $limit > 0 ) ? "<a href=\"".$hrefbase."&amp;limit=".$limit_prev.$todiv."\">«</a>&nbsp;" : "" ;
		$tcaption .= sprintf($voms_tcaption_tmpl, $limit+1, ( $limit_next < $allmembers ) ? $limit_next : $allmembers, $allmembers);
		if ( $limit_next < $allmembers ) $tcaption .= "&nbsp;<a href=\"".$hrefbase."&amp;limit=".$limit_next.$todiv."\">»</a>";
		echo $tcaption;
		echo "</div>\n";
	}

	// Add attribute form
	function ShowAddAtribute ( $vo_attributes, $gid = -1, $extra_hidden = "" ) {
		global $voms_attribute_caption, $voms_attribute_value_caption, $voms_attribute_add_submit, $voms_attributes_group_sel;
		echo "<form method=\"post\" action=\"#attrm\" >\n";
		if ( $gid !== -1 ) {
			printf("<label>%s</label><select name=\"agrp\">", $voms_attributes_group_sel );
			echo $gid;
			/*foreach ( getGroups() as $ggname => $ggid ) { 
				printf("<option value=\"%d\"%s>%s</option>\n", $ggid, ($gid == $ggid) ? " selected " : "", $ggname);
			}*/
			echo "</select><br/>\n";
		}
		printf("<label>%s</label><select name=\"attr\">", $voms_attribute_caption );
		$selected = ( isset($_POST["attr"]) ) ? $_POST["attr"] : ""; 
		foreach ( $vo_attributes as $attrid => $attrparm ) { 
			printf("<option value=\"%d\"%s>%s</option>\n",$attrid, 
				($selected == $attrid) ? " selected " : "", $attrparm["name"]);
		}
		echo "</select><br/>\n";
		printf("<label>%s</label>",$voms_attribute_value_caption);
		echo "<textarea name=\"attrval\" cols=\"40\" rows=\"5\" ></textarea><br/>\n";
		echo $extra_hidden;
		printf("<input class=\"ubtn\" type=\"submit\" name=\"attraddsub\" value=\"%s\" />\n", $voms_attribute_add_submit);
		echo "</form>";
	}

	// Add new member to VO
	function newMember ($crdn, $crcn, $crca, $crmail, $vo) {
		global $voms_create_new_wrongparameter, $voms_create_new_failed, $voms_addca_notfound;
		global $regex_dn, $regex_cn, $regex_email;
		// Check for parameter correctness
		if ( ! preg_match($regex_dn, $crdn) ) {
			printf("<p class=\"error\">".$voms_create_new_wrongparameter."</p>", "DN");
			return 0;
		}
		if ( ! preg_match($regex_cn, $crcn) ) {
			printf("<p class=\"error\">".$voms_create_new_wrongparameter."</p>", "CN");
			return 0;
		}
		if ( ! preg_match($regex_dn, $crca) ) {
			printf("<p class=\"error\">".$voms_create_new_wrongparameter."</p>", "CA");
			return 0;
		}
		if ( ! preg_match($regex_email, $crmail) ) {
			printf("<p class=\"error\">".$voms_create_new_wrongparameter."</p>", "E-mail");
			return 0;
		}
		// If we are still here - then create user
		$create_result = _invoke_transactional_sql(1,'createUser', $crdn, $crca, $crcn, $crmail, $vo );
		if ( $create_result === 0 ) {
			printf("<p class=\"error\">%s</p>\n",$voms_create_new_failed);
			return 0;
		} else if ( $create_result === -1 ) {
			printf("<p class=\"error\">%s</p>\n",$voms_addca_notfound);
			return 0;
		}
		return 1;
	}

	// Create a new group in VO
	function newGroup ($crgrp, $crpgrp, $vo ) {
		global $voms_addgroup_gid_wrong, $voms_creategrp_group_wrong, $voms_creategrp_new_failed;
		global $regex_name;
		if ( ! is_numeric($crpgrp) ) {
			print "<p class=\"error\">".$voms_addgroup_gid_wrong."</p>";
			return 0;
		}
		if ( ! preg_match($regex_name, $crgrp) ) {
			print "<p class=\"error\">".$voms_creategrp_group_wrong."</p>";
			return 0;
		}
		if ( _invoke_transactional_sql(1,'createGroup', $crgrp, $crpgrp, $vo ) === 0 ) {
			echo "<p class=\"error\">".$voms_creategrp_new_failed."</p>";
			return 0;
		}
		return 1;
	}

	// Create new Role in VO
	function newRole ($crrole, $vo ) {
		global $voms_createrole_role_wrong, $voms_createrole_allready, $voms_createrole_new_failed;
		global $regex_name;
		if ( ! preg_match($regex_name, $crrole) ) {
			print "<p class=\"error\">".$voms_createrole_role_wrong."</p>";
			return 0;
		}
		$crrres = _invoke_transactional_sql(1,'createRole', $crrole, $vo );
		if ( $crrres == -1 ) {
			print "<p class=\"error\">".$voms_createrole_allready."</p>";
			return 0;
		}
		if ( ! $crrres ) {
			echo "<p class=\"error\">".$voms_createrole_new_failed."</p>";
			return 0;
		}
		return 1;
	}
	// Update user information ( CN and mail)
	function updUserInfo ( $cn, $mail, $uid ) {
		global $voms_upd_userinfo_wrong;
		global $regex_cn, $regex_email;
		// Check for correct parameters
		if ( ! preg_match($regex_cn, $cn) ) {
			printf("<p class=\"error\">".$voms_upd_userinfo_wrong."</p>", "CN");
			return 0;
		}
		if ( ! preg_match($regex_email, $mail) ) {
			printf("<p class=\"error\">".$voms_upd_userinfo_wrong."</p>", "E-mail");
			return 0;
		}
		if ( $uid == 0 ) return 0;
		// Call update SQL operation
		return _invoke_transactional_sql(1,'updateUserInfo', $uid, $cn, $mail);	
	}

	// Assing membership to group
	function asignUserGroup( $gid, $uid ) {
		global $voms_addgroup_gid_wrong, $voms_addgroup_allready, $voms_addgroup_failed;
		// Check for correct parameters
		if ( ! is_numeric($gid) ) {
			print "<p class=\"error\">".$voms_addgroup_gid_wrong."</p>";
			return 0;
		}
		if ( $uid == 0 ) return 0;
		$gadd_res = _invoke_transactional_sql(1,'addMembership',$gid, $uid, 0);
		if ( $gadd_res === -1 ) {
			print "<p class=\"error\">".$voms_addgroup_allready."</p>";
			return 0;
		}
		if ( $gadd_res ) return 1;
		print "<p class=\"error\">".$voms_addgroup_failed."</p>";
		return 0;
	}

	// Dissmiss membership in group
	function dissmissUserGroup( $gid, $uid ) {
		global $voms_addgroup_gid_wrong, $voms_delgroup_allready, $voms_delgroup_failed;
		// Check for correct parameters
		if ( ! is_numeric($gid) ) {
			print "<p class=\"error\">".$voms_addgroup_gid_wrong."</p>";
			return 0;
		}
		if ( $uid == 0 ) return 0;
		$gdel_res = _invoke_transactional_sql(1,'delMembership', $gid, $uid, 0);
		if ( $gdel_res === -1 ) {
			print "<p class=\"error\">".$voms_delgroup_allready."</p>";
			return 0;
		}
		if ( $gdel_res ) return 1;
		print "<p class=\"error\">".$voms_delgroup_failed."</p>";
		return 0;
	}

	// Assing Role in group
	function addUserRole( $gid, $rid, $uid ) {
		global $voms_addgroup_gid_wrong, $voms_addrid_rid_wrong, $voms_addrid_allready, $voms_addrid_failed;
		// Check for correct parameters
		if ( ! is_numeric($gid) ) {
			print "<p class=\"error\">".$voms_addgroup_gid_wrong."</p>";
			return 0;
		}
		if ( ! is_numeric($rid) ) {
			print "<p class=\"error\">".$voms_addrid_rid_wrong."</p>";
			return 0;
		}
		if ( $uid == 0 ) return 0;
		$radd_res = _invoke_transactional_sql(1,'addMembership', $gid, $uid, $rid);
		if ( $radd_res === -1 ) {
			print "<p class=\"error\">".$voms_addrid_allready."</p>";
			return 0;
		}
		if ( $radd_res ) return 1;
		print "<p class=\"error\">".$voms_addrid_failed."</p>";
		return 0;
	}

	// Dissmiss Role in group
	function delUserRole( $gid, $rid, $uid ) {
		global $voms_addgroup_gid_wrong, $voms_addrid_rid_wrong, $voms_delrid_allready, $voms_delrid_failed;
		// Check for correct parameters
		if ( ! is_numeric($gid) ) {
			print "<p class=\"error\">".$voms_addgroup_gid_wrong."</p>";
			return 0;
		}
		if ( ! is_numeric($rid) ) {
			print "<p class=\"error\">".$voms_addrid_rid_wrong."</p>";
			return 0;
		}
		if ( $uid == 0 ) return 0;
		$rdel_res = _invoke_transactional_sql(1,'delMembership', $gid, $uid, $rid);
		if ( $rdel_res === -1 ) {
			print "<p class=\"error\">".$voms_delrid_allready."</p>";
			return 0;
		}
		if ( $rdel_res ) return 1;
		print "<p class=\"error\">".$voms_delrid_failed."</p>";
		return 0;
	}

	// Show attributes table
	function showAttr ( $attrs, $admin = 0, $classid, $roleattr = 0 ) {
		global $voms_attributes_notset, $voms_attributes, $voms_attributes_group, $voms_attribute_delete;
		if ( $attrs === 0 ) { printf ("<p>".$voms_attributes_notset."</p>\n", $classid); }
		else {
			echo "<table>\n<tr>";
			if ( $roleattr ) printf("<th>%s</th>", $voms_attributes_group);
			echo "<th>".key($voms_attributes)."</th><th>".current($voms_attributes)."</th><th></th></tr>\n";
			$odd = 0;
			foreach ( $attrs as $attrid => $attrparm ) {
				if ( $roleattr ) if ( $attrparm["aclallowed"] == 0 ) continue;
				doOddtrtd($odd);
				if ( $roleattr ) {
					printf("%s</td><td>", $attrparm["group"]);
					if ( $attrparm["aclallowed"] < 2 ) $admin = 0;
					else $admin = 1; 
				}

				echo $attrparm["name"] . "</td><td>" . $attrparm["value"] . "</td><td>";
				
				if ( $admin ) {
					echo "<form name=\"delattr".$attrid."\" method=\"post\" action=\"#attrm\" >";
					if ( $roleattr != 0 ) {
						printf("<input type=\"hidden\" name=\"rmattrid\" value=\"%s\" />\n", $attrparm["aid"]);
						printf("<input type=\"hidden\" name=\"rmattrgid\" value=\"%s\" />\n", $attrparm["groupid"]);
						printf("<input type=\"hidden\" name=\"agrp\" value=\"%s\" />\n", $roleattr);
					} else printf("<input type=\"hidden\" name=\"rmattrid\" value=\"%s\" />\n", $attrid );
					echo "<input type=\"hidden\" name=\"rmattr\" value=1 />";
					echo "</form>";
					echo "<a href=\"javascript:document.delattr".$attrid.".submit();\">".$voms_attribute_delete."</a>";
				}
				echo "</td>\n</tr>\n";
			}
			echo "</table>";
		}
	}

	// Add attribute for user
	function addAttr($aid, $uid, $value, $wt = "u", $gid = 0 ) {
		global $voms_attribute_id_wrong, $voms_attribute_notuniq, $voms_attribute_add_failed;
		global $voms_user, $voms_group;
		if ( ! is_numeric($aid) ) {
			print "<p class=\"error\">".$voms_attribute_id_wrong."</p>";
			return 0;
		}
		if ( $uid == 0 ) return 0;

		if ( $wt === "r" ) {
			$aua_res = _invoke_transactional_sql(1,'addRoleAttribute', $aid, $uid, $gid, $value);
		} else $aua_res = _invoke_transactional_sql(1,'addAttribute', $aid, $uid, $value, $wt ); 
		if ( $aua_res === -1 ) {
			if ( $wt === "g" ) $classid = $voms_group; else
			$classid = $voms_user;
			
			printf("<p class=\"error\">".$voms_attribute_notuniq."</p>", $classid);
			return 0;
		}
		if ( $aua_res ) return 1;
		print "<p class=\"error\">".$voms_attribute_add_failed."</p>";
		return 0;
	}

	// Add attribute for user
	function delAttr($aid, $uid, $wt = "u", $gid = 0) {
		global $voms_attribute_id_wrong, $voms_attribute_del_failed;
		if ( ! is_numeric($aid) ) {
			print "<p class=\"error\">".$voms_attribute_id_wrong."</p>";
			return 0;
		}
		if ( $uid == 0 ) return 0;

		if ( $wt === "r" ) {
			$aua_res = _invoke_transactional_sql(1,'delRoleAttribute', $aid, $uid, $gid, $value);
		} else $aua_res = _invoke_transactional_sql(1,'delAttribute', $aid, $uid, $wt);

		if ( $aua_res ) return 1;
		print "<p class=\"error\">".$voms_attribute_del_failed."</p>";
		return 0;
	}

	// Delete general VO attribute 
	function delVOAttr( $aid ) {
		global $voms_attribute_id_wrong, $voms_attribute_del_failed;
		if ( ! is_numeric($aid) ) {
			print "<p class=\"error\">".$voms_attribute_id_wrong."</p>";
			return 0;
		}
		$dres = _invoke_transactional_sql(1,'delVOAttribute', $aid);
		if ( $dres ) return 1;
		print "<p class=\"error\">".$voms_attribute_del_failed."</p>";
		return 0;
	}
	
	// Add general VO attribute
	function addVOAttr( $attrname, $attrdescr, $attruniq ) {
		global $voms_attribute_name_wrong, $voms_attribute_name_notuniq, $voms_attribute_add_failed;
		global $regex_name;
		if ( ! preg_match($regex_name, $attrname) ) {
			print "<p class=\"error\">".$voms_attribute_name_wrong."</p>";
			return 0;
		}
		$ares = _invoke_transactional_sql(1,'addVOAttribute', $attrname, $attrdescr, $attruniq);
		if ( $ares === -1 ) {
			print "<p class=\"error\">".$voms_attribute_name_notuniq."</p>";
			return 0;
		}
		if ( $ares ) return 1;
		print "<p class=\"error\">".$voms_attribute_add_failed."</p>";
		return 0;
	}

	// get admin id by admin info
	function getAdminId ( $adminfo ) {
		// if admin already exists - return id
		$admid = checkAdminId ( $adminfo );
		if ( $admid ) return $admid;
		if ( _invoke_transactional_sql(1,'createAdmin',$adminfo) ) return $GLOBALS['pva_createadmin_insid'];
		return 0;
	}

	// get admin id by userid
	function getAdminByUsr( $uid ) {
		$userinfo = getUserInfo( $uid );
		$adminfo = array ( "dn" => $userinfo[0], "mail" => $userinfo[3], "caid" => $userinfo[4], "ca" => $userinfo[2] );
		return getAdminId ( $adminfo );
	}
 
	// Draw ACL permissions table
	function printPermissionsTable ($perm_hash){
		global $voms_acl_values_caption;
		foreach( $voms_acl_values_caption as $acl_field => $captarr ) {
			printf("<tr><td>%s</td>\n", $captarr[0]);
			printf("<td class=\"center\"><div>%s</div><div><input style=\"width: auto;\" type=\"checkbox\" name=\"%sr\" %s /></div></td>\n", $captarr[1], $acl_field, ( $perm_hash[$acl_field]["r"] ) ? "checked=\"checked\"": "");
			printf("<td class=\"center\"><div>%s</div><div><input style=\"width: auto;\" type=\"checkbox\" name=\"%sw\" %s /></div></td>\n", $captarr[2], $acl_field, ( $perm_hash[$acl_field]["w"] ) ? "checked=\"checked\"": "");
			if ( isset($perm_hash[$acl_field]["d"]) )
				printf("<td class=\"center\"><div>%s</div><div><input style=\"width: auto;\" type=\"checkbox\" name=\"%sd\" %s /></div></td>\n", $captarr[3], $acl_field, ( $perm_hash[$acl_field]["d"] ) ? "checked": "");
			else echo "<td></td>\n";
			echo "</tr>\n";
		}
	}

	// New ACL Form 
	function showNewACLForm($vo, $gid, $default = 0) {
		global $voms_acl_new_caption, $voms_acl_failed_info, $voms_acl_new_for, $voms_acl_new_grant, $voms_acl_propagate;
		global $voms_acl_new_vouser, $voms_acl_new_nonvouser, $voms_acl_new_role, $voms_acl_new_ingroup, $voms_acl_new_group, $voms_acl_new_anyauth, $voms_acl_new_submit;
		$context = getGroupById($gid);
		if ( $context === 0 ) { printf("<p class=\"error\">%s</p>", $voms_acl_failed_info); return 0; }

		printf("<p class=\"underborder\">%s</p>\n", sprintf($voms_acl_new_caption,$context));
		
		printf("<p>%s</p>\n", $voms_acl_new_for);
		
		printf ("<form method=\"post\" action=\"?vo=%s&amp;operation=groupinfo&amp;id=%d\">\n", $vo, $gid );
		printf ("<input type=\"hidden\" name=\"defacl\" value=\"%d\">\n", $default );

		echo "<table style=\"border-bottom: 0px;\">\n";
		// vo user
		printf("<tr>\n<td><input style=\"width: auto;\" type=\"radio\" name=\"entryname\" value=\"vouser\">%s</input></td>\n",$voms_acl_new_vouser);
		echo "<td><select name=\"vouserv\">\n";
		foreach ( getVOMembersCA() as $userarr ) printf("<option value=\"%d\">%s</option>\n", $userarr["id"], $userarr["dn"]);
		echo "</select></td>\n</tr>\n";
		// non-vo user
		printf("<tr class=\"trodd\">\n<td><input style=\"width: auto;\" type=\"radio\" name=\"entryname\" value=\"nonvouser\">%s</input></td>\n",$voms_acl_new_nonvouser);
		echo "<td><div><input type=\"text\" name=\"nonvouserdn\" /></div>";
		echo "<div><select name=\"nonvouserca\">\n";
		foreach ( getCAList() as $caid => $caname ) printf("<option value=\"%d\">%s</option>\n", $caid, $caname);
		echo "</select></div></td>\n</tr>\n";
		// role
		printf("<tr>\n<td><input style=\"width: auto;\" type=\"radio\" name=\"entryname\" value=\"role\">%s</input></td>\n",$voms_acl_new_role);
		echo "<td><select name=\"rolev\" style=\"width: auto;\">\n";
		foreach ( getRoles() as $rolename => $roleid ) printf("<option value=\"%s\">%s</option>\n", $rolename, $rolename);
		printf("</select>&nbsp;%s&nbsp;<select name=\"rgroupv\" style=\"width: auto;\">\n", $voms_acl_new_ingroup);
		foreach ( getGroups() as $groupname => $groupid ) printf("<option value=\"%s\">%s</option>\n", $groupname, $groupname);
		echo "</select></td>\n</tr>\n";
		// group
		printf("<tr class=\"trodd\">\n<td><input style=\"width: auto;\" type=\"radio\" name=\"entryname\" value=\"group\">%s</input></td>\n",$voms_acl_new_group);
		echo "<td><select name=\"groupv\">\n";
		foreach ( getGroups() as $groupname => $groupid ) printf("<option value=\"%s\">%s</option>\n", $groupname, $groupname);
		echo "</select></td></tr>\n";
		// any auth
		printf("<tr><td colspan=\"2\"><input style=\"width: auto;\" type=\"radio\" name=\"entryname\" value=\"anyauth\">%s</input></td></tr>\n",$voms_acl_new_anyauth);
		echo "</table>\n";

		printf("<p>%s</p>\n", $voms_acl_new_grant);
		
		echo "<table style=\"border-bottom: 0px;\">\n";
		printPermissionsTable(decodeACLPermissions(0));
		if ( ! $default ) printf("<tr>\n<td>%s</td><td><input style=\"width: auto;\" type=\"checkbox\" name=\"propagate\" /></td><td></td><td></td>\n</tr>\n",$voms_acl_propagate);
		echo "</table>\n";
		printf("<input class=\"crnewbt\" type=\"submit\" name=\"aclnewsub\" value=\"%s\" />\n", $voms_acl_new_submit);

		echo "</form>\n";
	}

	// Show ACL management
	function showACLmanagement ( $vo, $pvap, $gid, $rid = NULL ) {
		global $voms_normal_acl_caption, $voms_default_acl_caption, $voms_no_acl_found, $voms_acl_table_colnames;
		global $voms_acl_error, $voms_acl_edit, $voms_acl_delete, $voms_acl_create;
		$acls = array ( 0 => $voms_normal_acl_caption, 1 => $voms_default_acl_caption);
		foreach ( $acls as $df => $acaption ) {
			printf("<p>%s</p>", $acaption);
			$dfaclf = ($df) ? "d" : "w";
			if ( isACLallow($pvap,"acl",$dfaclf)) printf("<span style=\"float: right;\"><a href=\"?vo=%s&amp;operation=aclnew&amp;id=%d&amp;def=%d\">%s</a></span>",$vo, $gid, $df, $voms_acl_create);
			$aclid = getACLid($gid, $rid, $df);

			if ( ! $aclid ) echo $voms_no_acl_found; else {
				echo "<table style=\"margin-top: -40px;\">";
				$fcol = true;
				foreach ($voms_acl_table_colnames as $colname )
					if ( $fcol ) { 
						printf("<tr><th style=\"vertical-align: bottom; text-align: center; padding-bottom: 5px;\">
							%s</th>", $colname);
						$fcol = false;
					} else printf("<th><div class=\"vtext_cont\"><p class=\"vtext\">%s</p></div></th>", $colname);
				echo "<th></th><th></th></tr>";
	
				$odd = 0;
				$aclv = getACLvalues($aclid);
				if ( $aclv === 0 ) printf("<tr><td colspan=5>%s</td></tr>",$voms_acl_error ); else {
					foreach ( $aclv as $adm_dn => $acl_val_arr ) {
						doOddtrtd($odd);
						printf ("<div class=\"mmbacldn\">%s<div>\n", $adm_dn);
						printf ("<div class=\"mmbca\">%s</div></td>\n", $acl_val_arr["ca"]);
						printf ("<td class=\"mid\">%s</td>\n", $acl_val_arr["container"]);
						printf ("<td class=\"mid\">%s</td>\n", $acl_val_arr["membership"]);
						printf ("<td class=\"mid\">%s</td>\n", $acl_val_arr["acl"]);
						printf ("<td class=\"mid\">%s</td>\n", $acl_val_arr["attributes"]);
						printf ("<td class=\"mid\">%s</td>\n", $acl_val_arr["requests"]);
						printf ("<td class=\"mid\">%s</td>\n", $acl_val_arr["preferences"]);
						if ( isACLallow($pvap,"acl",$dfaclf)) {
							printf("<td><a href=\"?vo=%s&amp;operation=acledit&amp;id=%d&amp;adm=%s\">%s</a></td>",$vo, $aclid, $acl_val_arr["admid"], $voms_acl_edit);
							printf("<td><a href=\"?vo=%s&amp;operation=acldel&amp;id=%d&amp;adm=%s\">%s</a></td>", $vo, $aclid, $acl_val_arr["admid"], $voms_acl_delete);
						} else echo "<td></td><td></td>";
						echo "</tr>";
					}
				}
				echo "</table>";
			}
		}
	}

	// Show Edit ACL form
	function editACLmanagement ( $vo, $aclid, $admid, $pvap, $udn = 0, $uca = 0, $umember = 0 ) {
		global $voms_acl_edit_caption, $voms_acl_failed_info, $voms_acl_admin, $voms_acl_context, $voms_acledit_submit, $voms_acl_propagate;
		echo "<p class=\"underborder\">". $voms_acl_edit_caption . "</p>\n";

		$perm = getPermissions ( $aclid, $admid );
		if ( $perm == 0 ) { printf("<p class=\"error\">%s</p>", $voms_acl_failed_info); return 0; }
		$group = getACLGroup ( $aclid );
		if ( $group == 0 ) { printf("<p class=\"error\">%s</p>", $voms_acl_failed_info); return 0; }
		$adminfo = getAdminInfo ( $admid );
		if ( $adminfo == 0 ) { printf("<p class=\"error\">%s</p>", $voms_acl_failed_info); return 0; }
		$dfaclf = ($group["default"]) ? "d" : "w";

		if ( $group["gid"] != 1 ) {
			$g_perm = getProperUserACL($udn, $uca, $umember, $group["gid"]);
			if ( $g_perm !== 0 ) $pvap = decodeACLPermissions($g_perm);
		}
		if ( ! isACLallow($pvap,"acl",$dfaclf)) return -1;

		printf ("<form method=\"post\" action=\"?vo=%s&amp;operation=groupinfo&amp;id=%d\">",$vo,$group["gid"] );	
		printf ("<input type=\"hidden\" name=\"aclid\" value=\"%d\" />", $aclid);
		printf ("<input type=\"hidden\" name=\"admid\" value=\"%d\" />", $admid);
		printf ("<input type=\"hidden\" name=\"admid2\" value=\"%d\" />", rand(0,99));
		printf ("<input type=\"hidden\" name=\"defacl\" value=\"%d\">", $group["default"] );
		echo "<table style=\"border: 0px;\">\n";
		printf("<tr><td>%s</td><td colspan=\"3\" ><div class=\"mmbacldn\">%s</div><div class=\"mmbca\">%s</div></td></tr>\n", $voms_acl_admin, $adminfo["cn"], $adminfo["ca"] );
		printf("<tr><td>%s</td><td colspan=\"3\" ><div class=\"greenselected\">%s</div></td></tr>\n", $voms_acl_context, $group["gdn"] );

		$perm_hash = decodeACLPermissions($perm);
		printPermissionsTable($perm_hash);

		if ( ! $group["default"] ) printf("<tr><td>%s</td><td><input style=\"width: auto;\" type=\"checkbox\" name=\"propagate\" /></td><td></td><td></td></tr>",$voms_acl_propagate);
		echo "</table>";
		printf("<input class=\"crnewbt\" type=\"submit\" name=\"acleditsub\" value=\"%s\" />\n", $voms_acledit_submit);
		echo "</form>";
	}

	// Show Delete ACL form
	function deleteACLmanagement ( $vo, $aclid, $admid, $pvap, $udn = 0, $uca = 0, $umember = 0 ) {
		global $voms_acl_del_caption, $voms_aclrem_from_children, $voms_acl_failed_info, $voms_acl_admin, $voms_acl_context, $voms_acl_permissions, $voms_acldel_submit;
		echo "<p class=\"underborder\">". $voms_acl_del_caption . "</p>\n";

		$perm = getPermissions ( $aclid, $admid );
		if ( $perm == 0 ) { printf("<p class=\"error\">1%s</p>", $voms_acl_failed_info); return 0; }
		$group = getACLGroup ( $aclid );
		if ( $group == 0 ) { printf("<p class=\"error\">2%s</p>", $voms_acl_failed_info); return 0; }
		$adminfo = getAdminInfo ( $admid );
		if ( $adminfo == 0 ) { printf("<p class=\"error\">3%s</p>", $voms_acl_failed_info); return 0; }
		$dfaclf = ($group["default"]) ? "d" : "w";

		if ( $group["gid"] != 1 ) {
			$g_perm = getProperUserACL($udn, $uca, $umember, $group["gid"]);
			if ( $g_perm !== 0 ) $pvap = decodeACLPermissions($g_perm);
		}

		if ( ! isACLallow($pvap,"acl",$dfaclf)) return -1;

		printf ("<form method=\"post\" action=\"?vo=%s&amp;operation=groupinfo&amp;id=%d\">",$vo,$group["gid"] );
		printf ("<input type=\"hidden\" name=\"defacl\" value=\"%d\">", $group["default"] );
		printf ("<input type=\"hidden\" name=\"aclid\" value=\"%d\" />", $aclid);
		printf ("<input type=\"hidden\" name=\"admid\" value=\"%d\" />", $admid);
		printf ("<input type=\"hidden\" name=\"admid3\" value=\"%d\" />", rand(0,99));

		echo "<table style=\"border: 0px;\">\n";
		printf("<tr><td>%s</td><td><div class=\"mmbacldn\">%s</div><div class=\"mmbca\">%s</div></td></tr>\n", $voms_acl_admin, $adminfo["cn"], $adminfo["ca"] );
		printf("<tr><td>%s</td><td><div class=\"greenselected\">%s</div></td></tr>\n", $voms_acl_context, $group["gdn"] );
		printf("<tr><td>%s</td><td><div>%s</div></td></tr>\n", $voms_acl_permissions, $perm );
		
		if ( ! $group["default"] ) printf("<tr><td>%s</td><td><input style=\"width: auto;\" type=\"checkbox\" name=\"propagate\" /></td><td></td><td></td></tr>",$voms_aclrem_from_children);
		echo "</table>";
		printf("<input class=\"crnewbt\" style=\"width: auto;\" type=\"submit\" name=\"acldelsub\" value=\"%s\" />\n", $voms_acldel_submit);
		echo "</form>";
	}

	// Show user management
	function showUsermanagement ( $vo, $limit, $filter, $admin = 0, $member=0, $gid = 0, $hrefbase_plus = null, $todiv = "" ) {
		global $voms_create_user, $voms_mng_user_search, $voms_no_search, $voms_mng_user_caption, $voms_delete_confirm, $voms_user_del, $voms_no_users;
		global $voms_leavevo, $voms_leavevo_sure;
		$odd = 0;
		$vo_mmb = getVOMembersCA($limit, $filter, $gid);
		$allmembers = getVOMembersCount($filter, $gid);
		// Create user link for VO admins
		//#PVA0.4 And remove myself from VO link
		$createuser_link = null;
		if ( $member ) $createuser_link  = sprintf("<a class=\"right\" href=\"javascript:ynalert('%s','?vo=%s&amp;operation=leavevo')\" >%s</a>",$voms_leavevo_sure,$vo,$voms_leavevo);
		if ( $admin == 1 )  $createuser_link .= sprintf("<a class=\"%sright\" href=\"?vo=%s&amp;operation=create\" >%s</a>",($member)?"almost":"",$vo,$voms_create_user);
		// Search functionality
		showSearchInput($voms_mng_user_search, $createuser_link );
		// Printing membership table if members exists
		if ( $allmembers == 0 ) { 
			if ( ! $filter ) printf("<p>%s</p>", $voms_no_users); else
			printf ("<p>". $voms_no_search . "</p>\n", htmlspecialchars($filter) ); 
		} else {
			echo "<p style=\"padding: 0;\">". $voms_mng_user_caption . "</p>\n";
			echo "<table>\n";
			foreach ( $vo_mmb as $vommbe ) {
				doOddtrtd($odd);
				echo "<div class=\"mmbdn\"><a href=\"?vo=".$vo."&amp;operation=userinfo&amp;id=". $vommbe["id"] ."\">". $vommbe["cn"] ."</a></div>\n";
				echo "<div class=\"mmbca\">". $vommbe["ca"]."</div></td>\n";
				// Delete user link for admins
				if ( $admin == 1 ) {
					$dellink = "?vo=". $vo ."&amp;operation=delete&amp;limit=".$limit."&amp;id=". $vommbe["id"];
					$deltext = sprintf ($voms_delete_confirm, $vommbe["cn"]);
					echo "<td class=\"adm\"><a href=\"javascript:ynalert('".$deltext."','".$dellink."')\" >".$voms_user_del."</a></td>";
				}
				echo "</tr>";
			} 
			echo "</table>\n";
			// Caption under table provide navigation between pages
			$hrefbase = "?vo=".$vo;
			if ( $hrefbase_plus ) $hrefbase .= $hrefbase_plus;
			ShowLimitsCaption($hrefbase, $allmembers, $todiv );
		}
	}

	// Show Groups management
	function showGroupmanagement ( $vo, $limit, $filter, $admin = 0) {
		global $voms_create_grp, $voms_mng_group_search, $voms_no_groups, $voms_no_groups_search, $voms_mng_groups_caption, $voms_deletegrp_confirm, $voms_grp_del;
		$vo_groups = getGroups($limit, $filter);
		$all_groups = getGroupsCount($filter);
		// Create group link for VO admins
		$creategrp_link = null;
		if ( $admin == 1 ) $creategrp_link = sprintf("<a class=\"right\" href=\"?vo=%s&amp;operation=creategrp\" >%s</a>",$vo,$voms_create_grp);
		// Search functionality
		$hrefbase = "?vo=".$vo."&operation=groupslist";
		showSearchInput($voms_mng_group_search, $creategrp_link, $hrefbase );
		// Printing groups table is exists
		if ( $all_groups == 0 ) {
			if ( ! $filter ) printf("<p>%s</p>", $voms_no_groups); else
			printf ("<p>". $voms_no_groups_search . "</p>\n", htmlspecialchars($filter) );
		} else {
			echo "<p>". $voms_mng_groups_caption . "</p>\n";
			echo "<table>\n";
			$odd = 0;
			foreach ( $vo_groups as $vgname => $vgl ) {
				doOddtrtd($odd);
				printf("<div><a href=\"?vo=%s&amp;operation=groupinfo&amp;id=%d\">%s</a><div></td>\n",$vo,$vgl,$vgname);
				// Delete group link for admins
				if ( $admin == 1 ) {
					if ( $vgname === "/".$vo ) { echo "<td></td>\n"; } else {
						$dellink = sprintf("?vo=%s&amp;operation=deletegrp&amp;id=%d",$vo,$vgl);
						$deltext = sprintf ($voms_deletegrp_confirm, $vgname);
						printf("<td class=\"adm\"><a href=\"javascript:ynalert('%s','%s')\" >%s</a></td>\n",$deltext,$dellink,$voms_grp_del);
					}
				}
				echo "</tr>";
			}
			echo "</table>\n";
			// Caption under table provide navigation between pages
			ShowLimitsCaption( $hrefbase, $all_groups );
		}
	}

	// Show Roles management
	function showRolesmanagement ( $vo, $limit, $filter, $admin = 0) {
		global $voms_create_role, $voms_no_roles, $voms_no_roles_search, $voms_mng_roles_caption, $voms_mng_roles_search, $voms_deleterole_confirm, $voms_role_del;
		$vo_roles = getRoles($limit, $filter);
		$all_roles = getRolesCount($filter);
		// Create Role link for VO admins
		$createrole_link = null;
		if ( $admin == 1 ) $createrole_link = sprintf("<a class=\"right\" href=\"?vo=%s&amp;operation=createrole\" >%s</a>",$vo,$voms_create_role);
		// Search functionality
		$hrefbase = "?vo=".$vo."&operation=roleslist";
		showSearchInput($voms_mng_roles_search, $createrole_link, $hrefbase );
		// Printing roles table is exists
		if ( $all_roles == 0 ) {
			if ( ! $filter ) printf("<p>%s</p>", $voms_no_roles); else
			printf ("<p>". $voms_no_roles_search . "</p>\n", htmlspecialchars($filter) );
		} else {
			echo "<p>". $voms_mng_roles_caption . "</p>\n";
			echo "<table>\n";
			$odd = 0;
			foreach ( $vo_roles as $vrname => $vrl ) {
				doOddtrtd($odd);
				printf("<div><a href=\"?vo=%s&amp;operation=roleinfo&amp;id=%d\">%s</a><div></td>\n",$vo,$vrl,$vrname);
				// Delete group link for admins
				if ( $admin == 1 ) {
					$dellink = sprintf("?vo=%s&amp;operation=deleterole&amp;id=%d",$vo,$vrl);
					$deltext = sprintf ($voms_deleterole_confirm, $vrname);
					printf("<td class=\"adm\"><a href=\"javascript:ynalert('%s','%s')\" >%s</a></td>\n",$deltext,$dellink,$voms_role_del);
				}
				echo "</tr>";
			}
			echo "</table>\n";
			// Caption under table provide navigation between pages
			ShowLimitsCaption( $hrefbase, $all_roles );
		}
	}

	// Show attributes assigned to VO users
	function showAttrsmanagement ( $vo, $limit, $filter, $admin = 0 ) {
		global $voms_manage_attributes, $voms_mng_attr_search, $voms_no_attrs, $voms_no_attrs_search, $voms_attributes, $voms_attributes_dn;
		$attributes = getAllUserAttributes($limit, $filter);
		$all_attrs = getAllUserAttributesCount($filter);
		// Manage attrs classes link for VO admins
		$mattr_link = null;
		if ( $admin == 1 ) $mattr_link = sprintf("<a class=\"right\" href=\"?vo=%s&amp;operation=mattr\" >%s</a>",$vo,$voms_manage_attributes);
		// Search functionality
		$hrefbase = "?vo=".$vo."&operation=attrlist";
		showSearchInput($voms_mng_attr_search, $mattr_link, $hrefbase );
		// Table of attributes
		if ( $all_attrs == 0 ) {
			if ( ! $filter ) printf("<p>%s</p>", $voms_no_attrs); else
			printf ("<p>". $voms_no_attrs_search . "</p>\n", htmlspecialchars($filter) );
		} else {
			echo "<br/><table>\n";
			echo "<th>".key($voms_attributes)."</th><th>".current($voms_attributes)."</th><th>".$voms_attributes_dn."</th></tr>\n";
			foreach ( $attributes as $attrp ) {
				doOddtrtd($odd);
				printf("%s</td>", $attrp["attr"]);
				printf("<td>%s</td>", $attrp["attrv"]);
				
				echo "<td><div class=\"mmbdn\"><a href=\"?vo=".$vo."&amp;operation=userinfo&amp;id=". $attrp["usrid"] ."\">". $attrp["cn"] ."</a></div>\n";
				echo "<div class=\"mmbca\">". $attrp["ca"]."</div></td>\n";
				echo "</tr>";
			}
			echo "</table>\n";
			// Caption under table provide navigation between pages
			ShowLimitsCaption($hrefbase, $all_attrs );
		}
	}


	// Generate unique ID
	define("ALPHABET", "0123456789abcdefghijklmnopqrstuvwxyz");
	function generate_unique_id($length = array(8,4,4,4,12), $alph = ALPHABET)
	{
		$al = strlen($alph) - 1;
		$result = '';
		$separator = '';
		foreach( $length as $slen ) {
			$result .= $separator;
			for($i = 0; $i < $slen; $i++)
				$result .= $alph[rand(0,$al)];
			$separator = "-";
		}
		return $result;
	}

	// Add new membership request to database
	function addNewRequest($vo, $dn, $ca, $mail, $inst, $phone, $comments ) {
		// generate required fields
		$cn = CNfromDN($dn);
		$confirm_id = generate_unique_id();

		// record to database
		$GLOBALS['ins_id'] = 0;
		$exit_status = _invoke_transactional_sql(1,'createNewRequest',$dn,$ca,$cn,$mail,$inst,$phone,$comments,$confirm_id);
		if ( $GLOBALS['ins_id'] === 0 ) return 0;

		$confirm_id .= $GLOBALS['ins_id'];
		unset($GLOBALS['ins_id']);

		$reqinfo = array ( "dn" => $dn, "ca" => $ca, "mail" => $mail, "inst" => $inst, "phone" => $phone, "cmnt" => $comments);
		$base_url = "https://".$_SERVER["HTTP_HOST"] .
				preg_replace("/\/index.php/","",$_SERVER["SCRIPT_NAME"]) .
				"?vo=" . $vo;

		send_userconfirm_mail($vo, $base_url, $confirm_id, $cn, $mail);
		return 1;
	}

	// Show pending requests management
	function showPendingRequestsmanagement ( $vo, $admin = 0 ) {
		global $voms_pending_caption, $voms_pending_no, $voms_pending_app, $voms_pending_rej;
		$vo_reqs = getVOPendingRequests();
		// Printing membership table if members exists
		echo "<p class=\"underborder\">". $voms_pending_caption . "</p>\n";
		if ( $vo_reqs == 0 ) {
			printf ("<div>%s</div>", $voms_pending_no);
		} else {
			echo "<table>\n";
			$odd = 0;
			foreach ( $vo_reqs as $reqid => $vommbe ) {
				doOddtrtd($odd);
				echo "<div class=\"mmbdn\"><a href=\"?vo=".$vo."&amp;action=subscriptions&amp;operation=pendinginfo&amp;id=". $reqid ."\">". $vommbe["cn"] ."</a></div>\n";
				echo "<div class=\"mmbca\">". $vommbe["ca"]."</div></td>\n";
				// Delete user link for admins
				if ( $admin == 1 ) {
					$applink = "?vo=". $vo ."&amp;action=subscriptions&amp;operation=approve&amp;id=". $reqid;
					$rejlink = "?vo=". $vo ."&amp;action=subscriptions&amp;operation=reject&amp;id=". $reqid;
					printf("<td class=\"adm\"><a href=\"%s\" >%s</a></td>",$applink, $voms_pending_app);
					printf("<td class=\"adm\"><a href=\"%s\" >%s</a></td>",$rejlink, $voms_pending_rej);
				}
				echo "</tr>";
			}
			echo "</table>\n";
		}
	}

	// Show processed requests
	function showProcessedRequestsmanagement( $vo ) {
		global $voms_processed_caption, $voms_processed_no, $voms_processed_app, $voms_processed_rej;
		$vo_reqs = getVOProcessedRequests();
		echo "<p class=\"underborder\">". $voms_processed_caption . "</p>\n";
		if ( $vo_reqs == 0 ) {
			printf ("<div>%s</div>", $voms_processed_no);
		} else {
			echo "<table>\n";
			$odd = 0;
			foreach ( $vo_reqs as $reqid => $vommbe ) {
				doOddtrtd($odd);
				echo "<div class=\"mmbdn\"><a href=\"?vo=".$vo."&amp;action=subscriptions&amp;operation=processedinfo&amp;id=". $reqid ."\">". $vommbe["cn"] ."</a></div>\n";
				echo "<div class=\"mmbca\">". $vommbe["ca"]."</div></td>\n";
				if ( $vommbe["approved"] ) 
					printf("<td>%s</td>", $voms_processed_app);
				else
					printf("<td>%s</td>", $voms_processed_rej);
				echo "</tr>";
			}
			echo "</table>\n";
		}
	}

	// Show processed request info
	function showSubscriptionInfo ( $vo, $id, $processed = 0 ) {
		global $voms_reqinfo_caption, $voms_pending_info, $voms_processed_info_ststus, $voms_processed_info_evdate, $voms_processed_app, $voms_processed_rej, $voms_pending_apprej;
		echo "<p class=\"underborder\">". $voms_reqinfo_caption . "</p>\n";
		$reqinfo = getReqInfo ( $id, $processed );
		foreach ( $voms_pending_info as $prid => $prcapt ) {
			printf("<p>%s</p>", $prcapt);
			printf("<div class=\"pl20\">%s</div>", $reqinfo[$prid] );
		}
		if ( $processed == 1 ) {
			printf("<p>%s</p>", $voms_processed_info_ststus);
			if ( $reqinfo["status"] == 2 ) 
				printf("<div class=\"pl20\">%s</div>", $voms_processed_app );
			else
				printf("<div class=\"pl20\">%s</div>", $voms_processed_rej );
			printf("<p>%s</p>", $voms_processed_info_evdate);
			printf("<div class=\"pl20\">%s</div>", $reqinfo["evdate"]);
		} else if ( $processed == 0 ) {
			echo "<div class=\"mt20\"></div>";
			$applink = "?vo=". $vo ."&amp;action=subscriptions&amp;operation=approve&amp;id=". $id;
			$rejlink = "?vo=". $vo ."&amp;action=subscriptions&amp;operation=reject&amp;id=". $id;
			printf($voms_pending_apprej, $rejlink, $applink);
		}
	}

	// Reject request
	function rejectSubscription( $vo, $id ) {
		global $voms_subscription_reject_ok, $voms_subscription_reject_err, $voms_registration_rejected;
		if ( _invoke_transactional_sql(1,'changeRequestState', $vo, $id, 3) ) {
			$reqinfo = getReqInfo ( $id, 1 );
			send_usernotify_mail($vo, $reqinfo["cn"], $reqinfo["mail"], $voms_registration_rejected);
			printf("<p class=\"alldone\">%s<p>", $voms_subscription_reject_ok);
			
		} else printf("<p class=\"error\">%s<p>", $voms_subscription_reject_err);
	}

	// Approve request
	function approveSubscription ( $vo, $id ) {
		global $voms_subscription_approve_ok, $voms_subscription_approve_err, $voms_registration_approved, $voms_addca_notfound;
		$approve_result = _invoke_transactional_sql(1,'changeRequestState', $vo, $id, 2);
		if ( $approve_result > 0 ) {
			$reqinfo = getReqInfo ( $id, 1 );
			send_usernotify_mail($vo, $reqinfo["cn"], $reqinfo["mail"], $voms_registration_approved);
			printf("<p class=\"alldone\">%s<p>", $voms_subscription_approve_ok);
		} else if ( $approve_result == -1 ) {
			printf("<p class=\"error\">%s<p>", $voms_addca_notfound);
		} else printf("<p class=\"error\">%s<p>", $voms_subscription_approve_err);
	}

	// Get vomses string 
	function get_vomses ($recreate = 0) {
		global $vo, $vo_host, $vo_port, $vo_cert;
		$result = array();
		// get local server vomses string
		$vomses_name = "vomses_" . preg_replace('/\./','_',$vo_host);

		if ( ! isset($GLOBALS[$vomses_name]) || $recreate ) {
			$certsubject = preg_replace("/subject= /","",exec(sprintf("openssl x509 -in %s -noout -subject", $vo_cert)));
			$GLOBALS[$vomses_name] = sprintf("\"%s\" \"%s\" \"%s\" \"%s\" \"%s\"", $vo, $vo_host, $vo_port, $certsubject, $vo);
			_invoke_transactional_sql(1,'saveSettingsToDB',$vomses_name,$GLOBALS[$vomses_name]);
		}

		$result[] = $GLOBALS[$vomses_name];
		// get replicants vomses strings if exists
		$updators_arr = get_authorized_updators(2);
		if ($updators_arr)
		foreach ( $updators_arr as $upd ) {
			$vomses_name = "vomses_" . preg_replace('/\./','_',$upd['endpoint']);
			if ( isset($GLOBALS[$vomses_name]) ) $result[] = $GLOBALS[$vomses_name];
		}
		return $result;
	}

	// Show form for sending e-mail to VOMS server administrator
	function showmailadmform($DN,$user) {
		global $voms_sendmail_caption, $voms_sendmail_preamble, $voms_captcha_label;
		global $voms_sendmail_username, $voms_sendmail_email, $voms_sendmail_body, $voms_sendmail_submit;
		printf("<p class=\"underborder bold\" >%s</p>\n",$voms_sendmail_caption);
		foreach ( $voms_sendmail_preamble as $vrptext )
			printf("<p>%s</p>", $vrptext);
		echo "<form method=\"post\" action=\"\">\n";
		// User authorized by certificate - use this name, and no aditional checking requered
		printf("<label>%s</label><input type=\"text\" name=\"username\"%s/><br/><br/>\n",$voms_sendmail_username, 
			($DN !== 0)?sprintf(" disabled class=\"greenselected noborder\" value=\"%s\"", $user):sprintf(" value=\"%s\"",
			isset($_POST["username"])?$_POST["username"]:""));
		printf("<label>%s</label><input type=\"text\" name=\"mailfrom\" value=\"%s\"/><br/><br/>\n",$voms_sendmail_email, 
			isset($_POST["mailfrom"])?$_POST["mailfrom"]:"");
		printf("<label>%s</label><textarea cols=\"60\" rows=\"12\" name=\"mailtext\">%s</textarea><br/><br/>\n",$voms_sendmail_body, 
			isset($_POST["mailtext"])?$_POST["mailtext"]:"");
		if ($DN === 0) {
			// Use captcha if user is not authorized by cert
			printf("<label><a href=\"javascript:ChangeCaptcha();\" ><img id=\"captimg\" src=\"%s?%s=%s\" border=\"0\" alt=\"captcha\" /></a></label>", "./kcaptcha/index.php", session_name(), session_id());
			printf("<label>%s</label><input type=\"text\" name=\"captcha\" /><br/><br/>\n",$voms_captcha_label);
		}
		printf("<input class=\"crnewbt\" type=\"submit\" name=\"mailsub\" value=\"%s\" />\n", $voms_sendmail_submit );
		echo "</form>\n";
	}

	// Show form for sending requests to serve new VO by VOMS server
	function shownewvoform($DN, $CA) {
		global $voms_newvo_non_https, $voms_newvo_no_cert;
		global $voms_newvo_policy, $voms_newvo_caption;
		global $voms_newvo_voname, $voms_newvo_admdn, $voms_newvo_admca, $voms_newvo_admmail, $voms_newvo_defca, $voms_newvo_descr, $voms_newvo_homepg, $voms_newvo_rules,$voms_newvo_submit;
		printf("<p class=\"underborder bold\" >%s</p>\n",$voms_newvo_caption);
		foreach ( $voms_newvo_policy as $vrptext )
			printf("<p>%s</p>", $vrptext);
		if (isset ($_SERVER['HTTPS'])) {
			if ( ( $DN !== 0 ) && ( $CA !== 0 ) ) {
				echo "<form method=\"post\" action=\"\">\n";
				printf("<label>%s</label><input type=\"text\" name=\"voname\" value=\"%s\"/><br/><br/>\n",
					$voms_newvo_voname, isset($_POST["voname"])?$_POST["voname"]:"");
				printf("<label>%s</label><input type=\"text\" name=\"admindn\" disabled class=\"greenselected noborder\"  value=\"%s\"/><br/><br/>\n",
					$voms_newvo_admdn, $DN);
				printf("<label>%s</label><input type=\"text\" name=\"adminca\" disabled class=\"greenselected noborder\"  value=\"%s\"/><br/><br/>\n",
					$voms_newvo_admca, $CA);
				printf("<label>%s</label><input type=\"text\" name=\"admmail\" value=\"%s\"/><br/><br/>\n",
					$voms_newvo_admmail, isset($_POST["admmail"])?$_POST["admmail"]:"");
				printf("<label>%s</label><input type=\"text\" name=\"defca\" value=\"%s\"/><br/><br/>\n",
					$voms_newvo_defca, isset($_POST["defca"])?$_POST["defca"]:$CA);
				printf("<label>%s</label><input type=\"text\" name=\"descr\" value=\"%s\"/><br/><br/>\n",
					$voms_newvo_descr, isset($_POST["descr"])?$_POST["descr"]:"");
				printf("<label>%s</label><input type=\"text\" name=\"homepg\" value=\"%s\"/><br/><br/>\n",
					$voms_newvo_homepg, isset($_POST["homepg"])?$_POST["homepg"]:"");
				printf("<label>%s</label><input type=\"text\" name=\"rules\" value=\"%s\"/><br/><br/>\n",
					$voms_newvo_rules, isset($_POST["rules"])?$_POST["rules"]:"");
				printf("<input class=\"crnewbt\" type=\"submit\" name=\"newvosub\" value=\"%s\" />\n", $voms_newvo_submit );

				echo "</form>\n";
			} else printf("<p class=\"error\">%s</p>",$voms_newvo_no_cert);
		} else printf("<p class=\"error\">%s</p>",sprintf($voms_newvo_non_https, "https://".$_SERVER['HTTP_HOST'].$_SERVER['REQUEST_URI']));
	
	}

	// Constract mail body for new VO request (make it bash compatible to easyly source)
	function construct_newvo_request($name,$admdn,$admca,$admmail,$defca,$descr,$homepg,$rules) {
		$message  = "# REQUEST FOR VOMS RESOURCES FOR NEW VO\n\n";
		$message .= sprintf("VONAME=\"%s\"\n",$name);
		$message .= sprintf("ADMDN=\"%s\"\n",$admdn);
		$message .= sprintf("ADMCA=\"%s\"\n",$admca);
		$message .= sprintf("ADMMAIL=\"%s\"\n",$admmail);
		$message .= sprintf("DEFCA=\"%s\"\n",$defca);
		$message .= sprintf("DESCR=\"%s\"\n",$descr);
		$message .= sprintf("HOMEPAGE=\"%s\"\n",$homepg);
		$message .= sprintf("RULES_URL=\"%s\"\n",$rules);
		return $message;
	}

	// Add technical information to mail message text
	function add_techinfo(&$message, $from, $frommail, $DN, $message_line_prefix="" ) {
		// And sending user parameters not only in mail header (for file dumping and usability)
		$message = sprintf("%smail from:\t\t%s(%s)\n%suser DN (if present):\t%s\n\n%s", $message_line_prefix, $from, $frommail, $message_line_prefix, $DN, $message);

		$message = sprintf("%s\n\n\n%s----TECHNICAL INFORMATION----\n", $message, $message_line_prefix);
		// some _SERVER values should be interested
		$message = sprintf("%s%sREQUEST_TIME=\"%s\"\n", $message, $message_line_prefix, isset($_SERVER["REQUEST_TIME"])?$_SERVER["REQUEST_TIME"]:"not set");
		$message = sprintf("%s%sHTTP_REFERER=\"%s\"\n", $message, $message_line_prefix, isset($_SERVER["HTTP_REFERER"])?$_SERVER["HTTP_REFERER"]:"not set");
		$message = sprintf("%s%sHTTP_USER_AGENT=\"%s\"\n", $message, $message_line_prefix, isset($_SERVER["HTTP_USER_AGENT"])?$_SERVER["HTTP_USER_AGENT"]:"not set");
		$message = sprintf("%s%sREMOTE_ADDR=\"%s\"\n", $message, $message_line_prefix, isset($_SERVER["REMOTE_ADDR"])?$_SERVER["REMOTE_ADDR"]:"not set");
		$message = sprintf("%s%sREMOTE_HOST=\"%s\"\n", $message, $message_line_prefix, isset($_SERVER["REMOTE_ADDR"])?gethostbyaddr($_SERVER["REMOTE_ADDR"]):"not set");
	}

	// Place copy of  mail messages to configured folder
	function save_mail_filecopy ($fname, $text) {
		global $mail_filecopies_path;
		$filename = $mail_filecopies_path . "/" . $fname;
		if ($handle = fopen($filename, 'w')) {
			if (fwrite($handle, $text) === FALSE) return false;
			fclose($handle);
		} else return false;
		return true;
	}

	// Update vo config file information (obsolete in PVA >= 0.6)
	function update_config_ifexist($pattern,$replacement,&$config_content) {
		if ( preg_match($pattern,$config_content) ) {
			$config_content = preg_replace($pattern,$replacement,$config_content);
		} else {
			$config_content = preg_replace('/\?\>/',sprintf("    %s\n?".">",$replacement),$config_content);
		}
	}
	

	function update_vo_config_file($vo, $description, $mainurl, $rules_link, $ldefaultca) {
		global $pva_install_path, $vo_description, $vo_mainurl, $vo_rules_link, $defaultca;

		$config_path = sprintf("%s/conf/vomses/%s.conf",$pva_install_path,$vo);
		$config_content = file_get_contents($config_path);

		// vo description
		$pattern = '/\$vo_description\s*=\s*.*;/';
		$replacement = sprintf("\$vo_description=\"%s\";",addslashes($description));
		if ( isset($vo_description) ) {
			if ( empty($description) ) $replacement = "";
			update_config_ifexist($pattern,$replacement,$config_content);
		} else if ( ! empty($description) ) update_config_ifexist($pattern,$replacement,$config_content);

		// vo mainurl
		$pattern = '/\$vo_mainurl\s*=\s*.*;/';
		$replacement = sprintf("\$vo_mainurl=\"%s\";",addslashes($mainurl));
		if ( isset($vo_mainurl) ) {
			if ( empty($mainurl) ) $replacement = "";
			update_config_ifexist($pattern,$replacement,$config_content);
		} else if ( ! empty($mainurl)  ) update_config_ifexist($pattern,$replacement,$config_content);

		// vo rules link
		if ( empty($rules_link)) return false;
		$pattern = '/\$vo_rules_link\s*=\s*.*;/';
		$replacement = sprintf("\$vo_rules_link=\"%s\";",addslashes($rules_link));
		update_config_ifexist($pattern,$replacement,$config_content);
	
		// defaultca
		$pattern = '/\$defaultca\s*=\s*.*;/';
		$replacement = sprintf("\$defaultca=\"%s\";",addslashes($ldefaultca));
		if ( isset($defaultca) ) {
			if ( empty($ldefaultca) ) $replacement = "";
			update_config_ifexist($pattern,$replacement,$config_content);
		} else if ( ! empty($ldefaultca) ) update_config_ifexist($pattern,$replacement,$config_content);

		// write new config
		$fhandle = fopen($config_path,'w');
		if ( ! fwrite($fhandle,$config_content) ) return 0;
		return 1;
	}

	// Update VO configuration in database
	function update_vo_config($description, $mainurl, $rules_link, $ldefaultca) {
		if ( ! saveSettingsToDB('vo_rules_link',$rules_link) ) return 0;
		if ( ! saveSettingsToDB('defaultca',$ldefaultca) ) return 0;
		if ( ! saveSettingsToDB('vo_description',$description)) return 0;
		if ( ! saveSettingsToDB('vo_mainurl',$mainurl)) return 0;
		return 1;
	}

	// Replication interface
	function gen_AdjCode ($len=16) {
		$characters = '0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ';
		$code = ""; 
		for ($p = 0; $p < $len; $p++) {
			$code .= $characters[mt_rand(0, strlen($characters)-1)];
    		}
		return $code;
	}

	function gen_ReplicationStatusString ($r_arr, $pattern) {
		global $voms_preferences_repl_status_arr;
		global $replication_interval;
		if ( ! isset($replication_interval) ) $replication_interval = 30;
		$n = $r_arr['status'];
		$ss = check_lastsync_time($r_arr['sync_diff'], $replication_interval);	

		$status_s = $voms_preferences_repl_status_arr[$n];
		$status_style = "";
		if ($n == 0) {
			$status_style = "error";
		} else if ($n == 2) {
			$status_s .= $r_arr['sync_time'];
			if ( $ss ) $status_style = "greenselected";
			else $status_style = "error";
		}
		return sprintf($pattern,$status_style,$status_s);
	}

	function get_rpc_url($endpoint, $vo, $rpc, $code=0, $ltt=0) {
		$res = sprintf("https://%s/voms/rpc.php?vo=%s&rpc=%s",$endpoint,$vo,$rpc);
		if ( $code ) $res .= sprintf("&code=%s",$code);
		if ( $ltt )  $res .= sprintf("&ltt=%s",$ltt);
		return $res;
	}

	function get_ca_hash ($ca_dn) {
		global $ca_certificates_path, $regex_ca_hash;
		$ca_hash_cmd = sprintf("cd %s && for i in `ls -1 *.0`; do if [ -n \"`openssl x509 -in \$i -noout -subject | grep %s`\" ]; then echo \$i; break; fi; done", $ca_certificates_path, escapeshellarg($ca_dn));
		$hash = exec($ca_hash_cmd);
		if ( ! preg_match($regex_ca_hash, $hash) ) return false;
		return $hash;
	}

	function get_data_from_url ($url, $port, $cahash, $err_subsys = 0) {
		global $ca_certificates_path, $voms_preferences_repl_curl_failed;
		$curl_h = curl_init();
		$curl_opts = array( CURLOPT_URL	=> $url,
				CURLOPT_PORT	=> $port,
				CURLOPT_HEADER	=> false,
				CURLOPT_USERAGENT	=> "PVARPCAgent",
				CURLOPT_RETURNTRANSFER	=> true,
				CURLOPT_SSL_VERIFYPEER	=> true,
				CURLOPT_SSL_VERIFYHOST	=> 2,
				CURLOPT_CAINFO	=> $ca_certificates_path . "/" . $cahash,
				CURLOPT_VERBOSE	=> false,
				CURLOPT_CONNECTTIMEOUT	=> 1,
				CURLOPT_TIMEOUT	=> 1
		);
		curl_setopt_array($curl_h,$curl_opts);
		$result = curl_exec($curl_h);
		$err	= curl_errno($curl_h); 
		$errmsg = curl_error($curl_h) ; 
		curl_close($curl_h);
		if ( $err ) {
			if ( ! $err_subsys ) printf("<p class=\"error\">%s%s</p>",$voms_preferences_repl_curl_failed, $errmsg);
			else storeLogRecord($err_subsys,1,array($errmsg));
			return 0;
		} else return $result;
	}

	function decode_rpc_result(&$result, $err_subsys = 0 ) {
		global $voms_rpc_error_caption, $voms_rpc_error_codes_descr, $voms_log_msg_codes;
		global $regex_pva_rpc;
		$rpc_rc = strtok($result,"\n");
		if ( ! preg_match($regex_pva_rpc, $rpc_rc)) {
			if ( $err_subsys ) storeLogRecord($err_subsys,5,array(htmlspecialchars($result)));
			else printf("<p class=\"error\">%s</p>",sprintf($voms_log_msg_codes[5],htmlspecialchars($result)));
			return 6;
		}
		$value = strtok("\n");
		$result = json_decode($value,true);
		$numeric_rpc_rc = strtok($rpc_rc, " ");
		switch ($numeric_rpc_rc) {
			case 0:
				return 0;
			case 1:
			case 2:
			case 3:
			case 4:
			case 5:
			case 6:
				if ( $err_subsys ) { 
					echo "storeLogRecord($err_subsys, 2,$numeric_rpc_rc, $voms_rpc_error_codes_descr[$numeric_rpc_rc])";
					storeLogRecord($err_subsys, 2, 
					array($numeric_rpc_rc, $voms_rpc_error_codes_descr[$numeric_rpc_rc]));
				}else printf("<p class=\"error\"><b>%s</b>%s</p>",$voms_rpc_error_caption, 
					$voms_rpc_error_codes_descr[$numeric_rpc_rc]);
				break;
			default:
				if ( $err_subsys ) storeLogRecord($err_subsys, 2, array(255, $voms_rpc_error_codes_descr[255]));
				else printf("<p class=\"error\"><b>%s</b>%s</p>",$voms_rpc_error_caption,
					$voms_rpc_error_codes_descr[255]);
		}
		return $numeric_rpc_rc;
	}

	function check_lastsync_time ($sync_diff, $interval, $allowed_fail_count = 3) {
		global $regex_digits, $regex_intervalh;	
		if ( preg_match($regex_intervalh, $interval) ) {
			$htime = (int) preg_replace('/h/','',$interval);
			$interval = 60 * $htime;
		} else if ( ! preg_match($regex_digits, $interval) ) $interval = 0;
		$interval *= 60;
		$interval *= $allowed_fail_count;
		if ( $sync_diff > $interval ) return 0;
		return 1;
	}

	function show_replication_adj_details ($id, $pvap) {
		global $vo, $voms_preferences_repl_adj_detail_caption,$voms_preferences_repl_adj_code,$voms_preferences_repl_endpoint,$voms_preferences_repl_endpointip,$voms_preferences_repl_code, $voms_preferences_repl_adj_bad_id, $voms_preferences_repl_adj_detail_status, $voms_preferences_repl_edit, $voms_preferences_repl_submit, $voms_preferences_repl_adj_confirm_req, $voms_preferences_repl_adj_confirm_req_text, $voms_preferences_repl_adj_confirm_checkbox, $voms_preferences_repl_adj_confirm_button, $voms_preferences_repl_adj_remove, $voms_preferences_repl_adj_remove_warn;
		$edit = _ispost("edit");
		if ( ! isACLallow($pvap,"preferences","r")) return 0;
		$adj_info =  get_authorized_updator_by_id($id);
		if ( ! $adj_info ) {
			printf("<p class=\"error\">%s</p>",$voms_preferences_repl_adj_bad_id);
			return 0;
		}

		printf("<p>&raquo;&nbsp;%s<b>%s</b></p>\n",$voms_preferences_repl_adj_detail_caption, $adj_info['dn']);

		if ( isACLallow($pvap,"preferences","w")) {
			echo "<form name=\"arm\" method=\"post\" action=\"\">\n";
			echo "<input type=\"hidden\" name=\"adj_delete\" value=\"1\"/>\n";
			echo "<input type=\"hidden\" name=\"adj_id\" value=\"".$id."\"/>\n";
			echo "</form>\n";
			$delete_url = sprintf("<a href=\"javascript:ynalert('%s','%s');\">%s</a>", $voms_preferences_repl_adj_remove_warn, "javascript:document.arm.submit()", $voms_preferences_repl_adj_remove);
			printf("<label>%s:</label><input type=\"text\" disabled class=\"greenselected noborder small\" name=\"adj_code_d\" value='%s' />(%s)<br/><br/>\n", $voms_preferences_repl_adj_code, $adj_info['auth_key'], $delete_url);
		} else {
			printf("<label>%s:</label><input type=\"text\" disabled class=\"greenselected noborder\" name=\"adj_code_d\" value='%s' /><br/><br/>\n", $voms_preferences_repl_adj_code, $adj_info['auth_key']);
		}

		printf("<label>%s:</label><input type=\"text\" disabled class=\"greenselected noborder\" name=\"repl_endpoint\" value='https://%s/voms/%s/' /><br/><br/>\n", $voms_preferences_repl_endpoint,$adj_info['endpoint'],$vo);
		printf("<label>%s:</label><input type=\"text\" disabled class=\"greenselected noborder\" name=\"repl_endpoint_ip\" value='%s' /><br/><br/>\n", $voms_preferences_repl_endpointip, $adj_info['ip']);
		printf("<label>%s:</label><input type=\"text\" disabled name=\"repl_code\" %s /><br/><br/>\n",$voms_preferences_repl_adj_detail_status, gen_ReplicationStatusString($adj_info,"class=\"noborder %s\" value='%s'"));

		if ( isACLallow($pvap,"preferences","w") ) {
			if ( $edit ) {
				echo "<form name=\"anrw\" method=\"post\" action=\"\">\n";
				$editlink = sprintf("<a href=\"javascript:document.anrw.submit();\">%s</a>",$voms_preferences_repl_submit);
				printf("<label>%s:</label><input type=\"text\" name=\"repl_code\" value='%s' /> (%s)<br/><br/>\n",$voms_preferences_repl_code,$adj_info['foreign_key'],$editlink);
				echo "<input type=\"hidden\" name=\"edit_repl_code\" value=\"1\"/>\n";
				echo "</form>\n";
			} else {
				echo "<form name=\"anrw\" method=\"post\" action=\"\">\n";
				echo "<input type=\"hidden\" name=\"edit\" value=\"1\"/>\n";
				echo "</form>\n";
				$editlink = sprintf("<a href=\"javascript:document.anrw.submit();\">%s</a>",$voms_preferences_repl_edit);
				printf("<label>%s:</label><input type=\"text\" disabled class=\"noborder small\" name=\"repl_code\" value='%s' />(%s)<br/><br/>\n",$voms_preferences_repl_code,$adj_info['foreign_key'], $editlink);
			}
		} else {
			printf("<label>%s:</label><input type=\"text\" disabled class=\"noborder small\" name=\"repl_code\" value='%s' /><br/><br/>\n",$voms_preferences_repl_code,$adj_info['foreign_key']);
		}

		if (( $adj_info['status'] == 0 ) && isACLallow($pvap,"preferences","w")) {
			printf("<p class=\"underborder\">&raquo;&nbsp;%s</p>\n",$voms_preferences_repl_adj_confirm_req);
			foreach ( $voms_preferences_repl_adj_confirm_req_text as $text ) 
				printf("<p>%s</p>",$text);
			echo "<form name=\"acc\" method=\"post\" action=\"\">\n";
			printf("<input type=\"checkbox\" name=\"adjconsumer\" style=\"width: auto;\" %s />%s<br/>\n",
				isset($_POST["adjconsumer"])?"checked=\"checked\"":"",$voms_preferences_repl_adj_confirm_checkbox);
			printf("<input class=\"crnewbt\" type=\"submit\" name=\"adjconfirm\" value=\"%s\" />\n",
				$voms_preferences_repl_adj_confirm_button);
			echo "</form>\n";
		}
	}

	function confirm_replicant ($id) {
		global $voms_preferences_repl_adj_bad_id, $voms_preferences_repl_consumer_failed, $voms_preferences_repl_consumer_success, $voms_preferences_repl_remote_ok, $voms_preferences_repl_bad_remote_status, $voms_preferences_repl_adj_enabled;
		global $vo, $regex_digits;

		if ( ! preg_match($regex_digits, $id)) {
			printf("<p class=\"error\">%s</p>",$voms_preferences_repl_adj_bad_id);
			return 0;
		}
		$isconsumer = isset($_POST["adjconsumer"]);
		$adj_info =  get_authorized_updator_by_id($id);
		if ( ! $adj_info ) {
			printf("<p class=\"error\">%s</p>",$voms_preferences_repl_adj_bad_id);
			return 0;
		}
		$fqdn = strtok($adj_info["endpoint"],":");
		$port = strtok(":");
		if ( ! $port ) $port = "443";
	
		// if PVA is explicitely consumer - get all database and invoke database rewrite
		if ( $isconsumer ) {
			$url = get_rpc_url($adj_info["endpoint"], $vo, "get_alldata", $adj_info["foreign_key"]);
			$rpcdata = get_data_from_url ($url, $port, $adj_info["cahash"]);
			if ( $rpcdata ) if ( ! decode_rpc_result($rpcdata) ) {
				if ( set_all_vo_tables($rpcdata,$id) ) {
					get_vomses(1);
					printf("<p class=\"alldone\">%s%s</p>",
						$voms_preferences_repl_consumer_success, $voms_preferences_repl_adj_enabled);
				} else printf("<p class=\"error\">%s</p>",$voms_preferences_repl_consumer_failed);
			}
		} else {
			// check connectivity and retreive replicant adjacency status
			$url = get_rpc_url($adj_info["endpoint"], $vo, "get_status", $adj_info["foreign_key"]);
			$rpcdata = get_data_from_url ($url, $port, $adj_info["cahash"]);
			if ( $rpcdata ) if ( ! decode_rpc_result($rpcdata) ) {
				// check if my instance already have replication active
				$this_status = get_replication_status(1);
				if ( $this_status ) {
					// confirm another replicants
					set_replication_status ($id, 1);
					get_vomses(1);
					printf("<p class=\"alldone\">%s</p>",$voms_preferences_repl_remote_ok);
				} else {
					// confirm if replicant alredy confirmed our adjacency
					if ( $rpcdata === 1 ) {
						set_replication_status ($id, 1);
						get_vomses(1);
						printf("<p class=\"alldone\">%s%s</p>",
							$voms_preferences_repl_remote_ok, $voms_preferences_repl_adj_enabled);
					} else printf("<p class=\"error\">%s</p>",$voms_preferences_repl_bad_remote_status);
				}
			}
		}
	}

	function show_replicants_table () {
		global $vo, $voms_preferences_repl_adj_list;
		printf("<p class=\"underborder bold\">%s</p>\n", $voms_preferences_repl_adj_list);
		$updators_arr = get_authorized_updators();
		echo "<table>\n";
		$odd = 0;
		foreach ( $updators_arr as $upd ) {
			doOddtrtd($odd, 420);
			printf("<div><a href=\"?vo=%s&action=preferences&operation=replicate&id=%d\">%s</a></div></td>\n",
				$vo,$upd['au_id'],$upd['dn']);
			printf("<td>%s</td>",gen_ReplicationStatusString($upd,"<p class=\"%s\">%s</p>"));
			echo "</tr>";
		}
		echo "</table>\n";
	}

	function show_add_new_replicant_wizard ($repl_dn="", $repl_endp="",$adj_code="",$repl_code="",$repl_ca="") {
		global $vo, $voms_preferences_repl_addnew, $voms_preferences_repl_addnew_descr;
		global $voms_preferences_repl_dn, $voms_preferences_repl_ca, $voms_preferences_repl_endpoint, $voms_preferences_repl_adj_code, $voms_preferences_repl_code, $voms_preferences_repl_new_adj_submit;
		if ( $adj_code == "" ) $adj_code = gen_AdjCode();
		printf("<span>&raquo;&nbsp;%s</span>\n",$voms_preferences_repl_addnew);
		foreach ( $voms_preferences_repl_addnew_descr as $msg )
			printf("<p>%s</p>\n",$msg);
		echo "<form name=\"anrw\" method=\"post\" action=\"\">\n";
		printf("<label>%s:</label><input type=\"text\" disabled class=\"greenselected noborder\" name=\"adj_code_d\" value='%s' /><br/><br/>\n",
			$voms_preferences_repl_adj_code,$adj_code);
		printf("<label>%s:</label><input type=\"text\" name=\"repl_dn\" value='%s' /><br/><br/>\n",
			$voms_preferences_repl_dn,$repl_dn);
		printf("<label>%s:</label><input type=\"text\" name=\"repl_ca\" value='%s' /><br/><br/>\n",
			$voms_preferences_repl_ca,$repl_ca);
		printf("<label>%s:</label><span class=\"greenselected\">https://<input class=\"small\" type=\"text\" name=\"repl_endpoint\" value='%s' />/voms/%s/</span><br/><br/>\n",
			$voms_preferences_repl_endpoint,$repl_endp,$vo);
		printf("<label>%s:</label><input type=\"text\" name=\"repl_code\" value='%s' /><br/><br/>\n",
			$voms_preferences_repl_code,$repl_code);
		printf("<input type=\"hidden\" name=\"adj_code\" value=\"%s\" />\n",$adj_code);
		printf("<input class=\"crnewbt\" type=\"submit\" name=\"new_adj\" value=\"%s\" />\n",$voms_preferences_repl_new_adj_submit);
		echo "</form>\n";
	}

	function add_new_replicant($repl_dn,$repl_endp,$adj_code,$repl_code,$repl_ca){
		global $voms_preferences_repl_new_wrongsyntax, $voms_preferences_repl_dn, $voms_preferences_repl_ca, $voms_preferences_repl_adj_code, $voms_preferences_repl_code, $voms_preferences_repl_endpoint, $voms_preferences_repl_resolve_failed, $voms_preferences_repl_curl_failed, $voms_preferences_repl_ping_filed, $voms_preferences_repl_dbfailed, $voms_preferences_repl_ca_nohash, $voms_preferences_repl_dnexists;
		global $vo, $regex_dn, $regex_code, $regex_socket, $regex_ip;
		// Check for parameter correctness
		if ( ! preg_match($regex_dn, $repl_dn) ) {
			printf("<p class=\"error\">".$voms_preferences_repl_new_wrongsyntax."</p>", $voms_preferences_repl_dn);
			return 0;
		}
		if ( ! preg_match($regex_dn, $repl_ca) ) {
			printf("<p class=\"error\">".$voms_preferences_repl_new_wrongsyntax."</p>", $voms_preferences_repl_ca);
			return 0;
		}
		$cahash = get_ca_hash($repl_ca);
		if ( ! $cahash ) {
			printf("<p class=\"error\">%s</p>",$voms_preferences_repl_ca_nohash);
			return 0;
		}
		if ( ! preg_match($regex_socket, $repl_endp) ) {
			printf("<p class=\"error\">".$voms_preferences_repl_new_wrongsyntax."</p>", $voms_preferences_repl_endpoint);
			return 0;
		} else {
			// determine IP
			$fqdn = strtok($repl_endp,":");
			$port = strtok(":"); 
			if ( ! $port ) $port = "443";
	
			if ( preg_match($regex_ip, $fqdn) ) {
				$ip = $fqdn;
			} else {
				$ip = gethostbyname($fqdn);
				if ( $ip === $fqdn ) {
					printf("<p class=\"error\">".$voms_preferences_repl_resolve_failed."</p>",$fqdn);
					return 0;
				}
				// After discussions I suggest backresolve checking is not required
				//if ( $fqdn !== gethostbyaddr($ip)) { }
			}
			// check RPC presence using CURL
			$url = get_rpc_url($repl_endp,$vo,"ping");
			$res = get_data_from_url($url,$port,$cahash);
			if ( ! $res ) return 0;
			if ( trim($res) !== "0 OK" ) {
				printf("<p class=\"error\">%s%s</p>",$voms_preferences_repl_curl_failed, $voms_preferences_repl_ping_filed);
				return 0;
			}
		}

		if ( ! preg_match($regex_code, $adj_code) ) {
			// if someone change POST manualy
			printf("<p class=\"error\">".$voms_preferences_repl_new_wrongsyntax."</p>", $voms_preferences_repl_adj_code);
			return 0;
		}
		if ( ! preg_match($regex_code, $repl_code) ) {
			printf("<p class=\"error\">".$voms_preferences_repl_new_wrongsyntax."</p>", $voms_preferences_repl_code);
			return 0;
		}
	
		// check if repl_dn allready exists
		$updators_arr = get_authorized_updators();
		if ($updators_arr) foreach ( $updators_arr as $upd ) {
			if ( $upd['dn'] == $repl_dn ) {
				printf("<p class=\"error\">%s</p>", $voms_preferences_repl_dnexists);
				return 0;
			}
		}

		if ( add_authorized_updator ($repl_dn, $repl_endp, $ip, $adj_code, $repl_code, $cahash) ) return 1;
		else {
			printf("<p class=\"error\">%s</p>", $voms_preferences_repl_dbfailed);
			return 0;
		}
	}

	function update_replicant ( $id, $repl_code ) {
		global $voms_preferences_repl_new_wrongsyntax, $voms_preferences_repl_code, $voms_preferences_repl_adj_bad_id;
		global $regex_digits, $regex_code;

		if ( ! preg_match($regex_digits, $id)) {
			printf("<p class=\"error\">%s</p>",$voms_preferences_repl_adj_bad_id);
			return 0;
		}

		if ( ! preg_match($regex_code, $repl_code) ) {
			printf("<p class=\"error\">".$voms_preferences_repl_new_wrongsyntax."</p>", $voms_preferences_repl_code);
			return 0;
		}
		
		if ( update_replication_code($id, $repl_code) ) return 1;
		else {
			printf("<p class=\"error\">%s</p>", $voms_preferences_repl_dbfailed);
			return 0;
		}
	}

	function transactions_optimizer(&$tr_arr) {
		//TODO: Some methods to solve possible conflicts need to be implemented
		return 1;
	}

	function transactional_update () {
		global $vo;
		$updators = get_authorized_updators(1);
		if ( ! $updators ) return 0;
		$alltransactions = array ();
		foreach ( $updators as $adj_info ) {
			// get port
			$fqdn = strtok($adj_info["endpoint"],":");
			$port = strtok(":");
			if ( ! $port ) $port = "443";
			// form ltt url
			$url = get_rpc_url($adj_info["endpoint"], $vo, "get_ltt", $adj_info["foreign_key"]);
			$rpcdata = get_data_from_url ($url, $port, $adj_info["cahash"], 1);
			if ( $rpcdata ) if ( ! decode_rpc_result($rpcdata, 1) ) {
				$ntstamp = $rpcdata;
				$ttstamp = $adj_info["ut_stamp"];
				// if new transactions
				if ( $ntstamp > $ttstamp ) {
					// form tdiff url
					$url = get_rpc_url($adj_info["endpoint"], $vo, "get_tdiff", $adj_info["foreign_key"], $ttstamp);
					$rpcdata = get_data_from_url ($url, $port, $adj_info["cahash"], 1);
					if ( $rpcdata ) {
						if ( ! decode_rpc_result($rpcdata, 1) ) foreach ( $rpcdata as $trans_detail ) {
							$alltransactions[] = array ( $adj_info["au_id"], 
								$trans_detail["adminid"],
								$trans_detail["uuid"], 
								$trans_detail["fname"], 
								$trans_detail["args"],
								$adj_info["status"],
								$ntstamp );
						}
					} else update_transactions_sync_time($adj_info["au_id"]);
				} else update_transactions_sync_time($adj_info["au_id"]);
			}
		}
		transactions_optimizer($alltransactions);
		foreach ( $alltransactions as $transaction ) {
			call_user_func_array("_invoke_transactional_sql_update",$transaction);
		}
	}

	// operations for editing crontab entries
	function get_cron_time ($time) {
		global $regex_digits, $regex_intervalh;
		$minutes = "";
		$hours = "";
		$coma = "";
		srand((double) microtime() * 1000000);
		if ( preg_match($regex_digits, $time) ) {
			if ( $time > 59 ) return 0;
			for ( $i=rand(0,$time); $i<60; $i+=$time ) {
				$minutes .= $coma . $i;
				$coma = ",";
			}
			$hours = "*";
		} else if ( preg_match($regex_intervalh, $time) ) {
			$htime = (int) preg_replace('/h/','',$time);
			if ( $htime > 23 ) return 0;
			if ( ! isset($htime) ) $htime = 4;
			for ( $i=rand(0,$htime); $i<24; $i+=$htime ) {
				$hours .= $coma . $i;
				$coma = ",";
			}
			$minutes = rand(0,60);
		} else return 0;

		return sprintf("%s %s * * *",$minutes, $hours);
	}

	function del_transactions_cron ($vo) {
		global $pva_install_path;
		// removing with ex cannot remove last single line, so sed here to overcome :-)
		$exec_cmd = sprintf("crontab -l | sed '/php modules\/cron\.php %s ) 2>&1/d' | crontab 2>&1 >/dev/null", $vo);
		shell_exec($exec_cmd);
		return 1;
	}

	function set_transactions_cron ($vo, $time = 30) {
		global $pva_install_path;
		del_transactions_cron($vo);
		saveSettingsToDB('replication_interval',$time);
		$GLOBALS['replication_interval'] = $time;
		$exec_cmd = sprintf("echo -e '\$a\\n%s ( cd %s && php modules/cron.php %s ) 2>&1 > /dev/null\\n.\\nx' | EDITOR=ex crontab -e", get_cron_time($time), $pva_install_path, $vo);
		shell_exec($exec_cmd);
	}

	// get function description 
	function get_function_description ($fname, $fargs) {
		global $voms_functions_descriptions, $voms_user, $voms_group;
		$template = $voms_functions_descriptions[$fname];
		switch ($fname) {
			case 'createUser':
				return sprintf($template,CNfromDN($fargs[0]),CNfromDN($fargs[1]));
			case 'createRole':
				return sprintf($template,$fargs[0]);
			case 'createGroup':
				return sprintf($template,_isdef($fargs[3]['group_name']));
			case 'deleteUser':
				return sprintf($template,CNfromDN(_isdef($fargs[1]['userdn'])));
			case 'deleteRole':
				return sprintf($template,_isdef($fargs[2]['role_name']));
			case 'deleteGroup':
				return sprintf($template,_isdef($fargs[2]['group_name']));
			case 'updateUserInfo':
				return sprintf($template,_isdef($fargs[3]['user_dn']),$fargs[1],$fargs[2]);
			case 'updateACLPermissions':
				return sprintf($template,_isdef($fargs[6]['group_name']),_isdef($fargs[6]['admin_cn']),$fargs[2]);
			case 'deleteACLentry':
				if ( $fargs[2] ) $template .= $voms_functions_descriptions['deleteACLentryChilds'];
				return sprintf($template,_isdef($fargs[4]['admin_cn']),_isdef($fargs[4]['group_name']));
			case 'addMembership':
				if ( $fargs[2] ) {
					$template .= $voms_functions_descriptions['addMembershipRole'];
					return sprintf($template,CNfromDN(_isdef($fargs[3]['user_name'])),_isdef($fargs[3]['group_name']),
								_isdef($fargs[3]['role_name']));
				} else return sprintf($template,CNfromDN(_isdef($fargs[3]['user_name'])),_isdef($fargs[3]['group_name']));
			case 'delMembership':
				if ( $fargs[2] ) {
					$template .= $voms_functions_descriptions['addMembershipRole'];
					return sprintf($template,CNfromDN(_isdef($fargs[3]['user_name'])),_isdef($fargs[3]['group_name']),
								_isdef($fargs[3]['role_name']));
				} else return sprintf($template,CNfromDN(_isdef($fargs[3]['user_name'])),_isdef($fargs[3]['group_name']));
			case 'addAttribute':
				if ( $fargs[3] === "u" ){
					return sprintf($template,_isdef($fargs[4]['attr_name']),$fargs[2],
							$voms_user,_isdef($fargs[4]['user_name']));
				} else  return sprintf($template,_isdef($fargs[4]['attr_name']),$fargs[2],
							$voms_group,_isdef($fargs[4]['group_name']));
			case 'delAttribute':
				if ( $fargs[2] === "u" ){
					return sprintf($template,_isdef($fargs[3]['attr_name']),
							$voms_user,_isdef($fargs[3]['user_name']));
				} else  return sprintf($template,_isdef($fargs[3]['attr_name']),
							$voms_group,_isdef($fargs[3]['group_name']));
			case 'addRoleAttribute':
				return sprintf($template,_isdef($fargs[4]['attr_name']),$fargs[3],
							_isdef($fargs[4]['role_name']),_isdef($fargs[4]['group_name']));
			case 'delRoleAttribute':
				return sprintf($template,_isdef($fargs[3]['attr_name']),_isdef($fargs[3]['role_name']),
							_isdef($fargs[3]['group_name']));
			case 'delVOAttribute':
				return sprintf($template,_isdef($fargs[1]['attr_name']));
			case 'addVOAttribute':
				if ( $fargs[2] ) $template = $voms_functions_descriptions['addVOAttributeUniq'];
				return sprintf($template,$fargs[0],$fargs[1]);
			case 'createNewRequest':
				return sprintf($template,CNfromDN($fargs[0]),CNfromDN($fargs[1]));
			case 'confirmRegRequest':
				return sprintf($template,CNfromDN($fargs[1]),CNfromDN($fargs[2]));
			case 'deleteRegRequest':
				return sprintf($template,CNfromDN($fargs[1]),CNfromDN($fargs[2]));
			case 'changeRequestState':
				if ( $fargs[2] == 2 ) $template .= $voms_functions_descriptions['changeRequestStateA'];
				else $template .= $voms_functions_descriptions['changeRequestStateD'];
				return sprintf($template,CNfromDN(_isdef($fargs[3]['user_dn'])),CNfromDN(_isdef($fargs[3]['user_ca'])));
			case 'createAdmin':
				return sprintf($template,CNfromDN($fargs[0]['dn']),CNfromDN($fargs[0]['ca']));
			case 'saveSettingsToDB':
				return sprintf($template,$fargs[0],$fargs[1]);
			case 'update_vo_config':
				return sprintf($template,$fargs[0],$fargs[1],$fargs[2],$fargs[3]);
		}
	}
?>
