<?php
//
//    Copyright 2010 Andrii Salnikov
//
//   Licensed under the Apache License, Version 2.0 (the "License");
//   you may not use this file except in compliance with the License.
//   You may obtain a copy of the License at
//
//       http://www.apache.org/licenses/LICENSE-2.0
//
//   Unless required by applicable law or agreed to in writing, software
//   distributed under the License is distributed on an "AS IS" BASIS,
//   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//   See the License for the specific language governing permissions and
//   limitations under the License.
//                                   
//   PHP VOMS-Admin RPC interface
//

//--- globals
ini_set("include_path", "./modules:./lang:./content:./conf");
error_reporting(E_ALL); ini_set("display_errors", 1);
require("config.inc");

//--- functions
function RPC_Invalid ($code=255) {
	switch ($code) {
		case 1:
			echo "1 INVALID_VO";
			break;
		case 2: 
			echo "2 HTTPS_REQUIRED";
			break;
		case 3:
			echo "3 AUTH_NOT_GIVEN";
			break;
		case 4: 
			echo "4 AUTH_FAILED";
			break;
		case 5:
			echo "5 BAD_REQUEST";
			break;
		case 6:
			echo "6 FAILED_TO_PROCESS";
			break;
		default:
			echo "255 REQUEST_ERROR";
	}
	exit($code);
}

function RPC_OK ($responce) {
	echo "0 OK\n";
	switch (gettype($responce)) {
		case 'array':
			//print_r($responce);
			echo json_encode($responce);
			break;
		default:
			echo $responce;
	}
}

//--- determine VO
$vo = ( isset($_GET["vo"]) ) ? $_GET["vo"] : 0;
if ( $vo === 0 ) RPC_Invalid(1);
$conffile = $pva_install_path . "/conf/vomses/" . $vo . ".conf";
if ( is_readable($conffile) ) {
	include($conffile);
	require("sql.php");
	if ( ! getSettingFromDB() ) {
		createSettingTable();
	}
} else RPC_Invalid(1);

//--- get rpc request
$rpc = ( isset($_GET["rpc"]) ) ? $_GET["rpc"] : 0;
if ( $rpc === 0 ) RPC_Invalid(5);


//--- check authorization
if ( $rpc !== "ping" ) {
	if ( ! isset ($_SERVER['HTTPS'])) RPC_Invalid(2);
	if ( $_SERVER['HTTP_USER_AGENT'] != "PVARPCAgent" ) RPC_Invalid(4);

	$code = ( isset($_GET["code"]) ) ? $_GET["code"] : 0;
	if ( $code === 0 ) RPC_Invalid(3);

	$au_ip = $_SERVER['REMOTE_ADDR'];
	$au_id = get_authorized_updator_id($au_ip, $code);
	if ( ! $au_id ) RPC_Invalid(4);

	switch ($rpc) {
		case 'get_status':
			$aus = get_authorized_updator_status($au_id);
			if ($aus === -1) RPC_Invalid(6);
			RPC_OK($aus);
			break;
		case 'get_ltt':
			$ltt = get_last_transaction_time();
			if (! $ltt) RPC_Invalid(6);
			RPC_OK($ltt);
			break;
		case 'get_tdiff':
			$ltt = ( isset($_GET["ltt"]) ) ? $_GET["ltt"] : 0;
			if ( $ltt === 0 ) RPC_Invalid(5);
			$tdiff = get_transactions_diff($ltt);
			if ( $tdiff === 0 ) RPC_Invalid(6);
			RPC_OK($tdiff);
			break;
		case 'get_alldata':
			$all_data = get_all_vo_tables();
			if ( ! $all_data ) RPC_Invalid(6);
			RPC_OK(json_encode($all_data));
			break;
		default:
			RPC_Invalid(5);
	}
} else {
	echo "0 OK";
}

?>

